import * as i0 from '@angular/core';
import { NgModule } from '@angular/core';
import { generateAlias } from '@devs-studio/string';
import * as moment from 'moment';
import { validateNumber } from '@devs-studio/sunat/dist/table/table-2';

class DevsValidatorsModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.5", ngImport: i0, type: DevsValidatorsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.2.5", ngImport: i0, type: DevsValidatorsModule }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.2.5", ngImport: i0, type: DevsValidatorsModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.5", ngImport: i0, type: DevsValidatorsModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [],
                    exports: []
                }]
        }] });

function _checkDuplicates(inputs, fields, alias) {
    var items = inputs.reduce((acc, cur) => {
        // Verificar si cur es un objeto vacío
        if (Object.keys(cur).length === 0) {
            return acc; // Ignorar objetos vacíos
        }
        var parts = [];
        for (let field of fields) {
            if (cur[field] !== null && cur[field] !== undefined) {
                if (alias) {
                    parts.push(generateAlias(cur[field]));
                }
                else {
                    parts.push(cur[field]);
                }
            }
            else {
                return acc;
            }
        }
        acc.push(parts.join('|'));
        return acc;
    }, []);
    var uniques = items.filter((value, index, self) => {
        return self.indexOf(value) === index;
    });
    return items.length === uniques.length;
}
function checkArrayUnique(fields, alias) {
    return (control) => {
        const items = control.value;
        if (!_checkDuplicates(items, fields, alias)) {
            return { unique: { value: control.value } };
        }
        return null;
    };
}

function _baseFunction(right, rule, callback) {
    return (control) => {
        const leftValue = Number(control.value);
        const formGroup = control.parent;
        if (formGroup) {
            const rightValue = Number(formGroup.get(right)?.value);
            if (!Number.isNaN(leftValue) && !Number.isNaN(rightValue) && !callback(leftValue, rightValue)) {
                // formGroup.get(right)?.updateValueAndValidity({ emitEvent: false });
                return { [rule]: true };
            }
        }
        return null;
    };
}
function isGreaterThanOrEqual(right) {
    return _baseFunction(right, 'greaterThanOrEqual', (left, right) => {
        return left >= right;
    });
}
function isLessThanOrEqual(right) {
    return _baseFunction(right, 'lessThanOrEqual', (left, right) => {
        return left <= right;
    });
}

function mustMatchValidator(controlName, matchingControlName) {
    return (formGroup) => {
        const control = formGroup.controls[controlName];
        const matchingControl = formGroup.controls[matchingControlName];
        if (matchingControl.errors && !matchingControl.errors['confirmedValidator']) {
            return;
        }
        if (control.value !== matchingControl.value) {
            matchingControl.setErrors({ mustMatch: true });
        }
        else {
            matchingControl.setErrors(null);
        }
    };
}

function isEnum(entity) {
    return (control) => {
        const value = control.value;
        if (value && !Object.values(entity).includes(value)) {
            return { isEnum: { value: control.value } };
        }
        return null;
    };
}

function _isValidDate(format) {
    return (control) => {
        const value = control.value;
        if (value && !moment(value, format, true).isValid()) {
            return { 'isValidDate': true };
        }
        return null;
    };
}
function isValidDate(format) {
    return _isValidDate(format);
}

function isValidIdentificationNumber(code_attribute) {
    return (control) => {
        const value = control.value;
        const formGroup = control.parent;
        if (formGroup) {
            const code = formGroup.get(code_attribute)?.value;
            if (!validateNumber(code, value)) {
                return { isValidIdentificationNumber: { value: control.value } };
            }
        }
        return null;
    };
}

function _checkAtLeast(inputs, field, value, quantity = 1) {
    var count = inputs.reduce((acc, cur) => {
        if (cur[field] === value) {
            return acc + 1;
        }
        else {
            return acc;
        }
    }, 0);
    return count === quantity;
}
function checkArrayAtLeast(field, value, quantity = 1) {
    return (control) => {
        const items = control.value;
        if (!_checkAtLeast(items, field, value, quantity)) {
            return { atLeast: { value: control.value } };
        }
        return null;
    };
}

function _checkAtLeast2(inputs, fn, quantity = 1) {
    var count = inputs.reduce((acc, cur) => {
        if (fn(cur)) {
            return acc + 1;
        }
        else {
            return acc;
        }
    }, 0);
    return count === quantity;
}
function checkArrayAtLeast2(fn, quantity = 1) {
    return (control) => {
        const items = control.value;
        if (!_checkAtLeast2(items, fn, quantity)) {
            return { atLeast: { value: control.value } };
        }
        return null;
    };
}

class DevsValidators {
    static checkArrayUnique(fields, alias) {
        return checkArrayUnique(fields, alias);
    }
    static checkArrayAtLeast(field, value, quantity = 1) {
        return checkArrayAtLeast(field, value, quantity);
    }
    static checkArrayAtLeast2(fn, quantity = 1) {
        return checkArrayAtLeast2(fn, quantity);
    }
    static isGreaterThanOrEqual(right) {
        return isGreaterThanOrEqual(right);
    }
    static isLessThanOrEqual(right) {
        return isLessThanOrEqual(right);
    }
    static mustMatchValidator(controlName, matchingControlName) {
        return mustMatchValidator(controlName, matchingControlName);
    }
    static isEnum(entity) {
        return isEnum(entity);
    }
    static isValidDate(format) {
        return isValidDate(format);
    }
    static isValidIdentificationNumber(controlName) {
        return isValidIdentificationNumber(controlName);
    }
}

/*
 * Public API Surface of validators
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DevsValidators, DevsValidatorsModule };
//# sourceMappingURL=devs-studio-angular-validators.mjs.map
